function flag = datacheck_nomean(H, r)
% Check if the matrix factorization Y -> U V' can uniquely be performed (up
% to the natural ambiguity of the factorization) from the associated
% missing-component-indicator matrix H. 
%
% flag = datacheck_nomean(H, r) yields 0 if the factorization is possibly
% non-unique and 1 otherwise. r specifies the column size of U and V of Y
% -> U V'. The function examines whether the rank of QFG (see the
% literature) is equal to (n-r)*r or not.  

m = size(H,1);
n = size(H,2);

% Generate u and v according to Gaussian dist.
u = randn(m*r,1);
v = randn(n*r,1);

% Count the nonmissing components
p = 0;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, p = p + 1;
        end
    end
end
F = zeros(p, m*r);
G = zeros(p, n*r);

% Make F from v
q = 1;
for i=1:m
    for j=1:n
        if H(i, j) > 0.5, 
            F(q, (i-1)*r+1:(i-1)*r+r) = v((j-1)*r+1:(j-1)*r+r);
            q = q + 1;
        end
    end
end

% Make G from u
q = 1; 
for i = 1:m
    for j = 1:n
        if H(i, j) > 0.5,
            G(q, (j-1)*r+1:(j-1)*r+r) = u((i-1)*r+1:(i-1)*r+r);
            q = q + 1;
        end
    end
end

% Compute QFG
FFTFinv = F*inv(F.'*F);
QFG = G - FFTFinv*(F.'*G);

% Check the rank of QFG

%[U1, W1, V1] = svd(QFG);

if rank(QFG) ~= (n-r)*r, 
    disp('Incomplete data: The solution will not be uniquely determined unless some of the nonmissing components need to be additionally nonmissing.');
    flag = 0;
else
    flag = 1;
end
