function [U, V, mu] = wiberg_mean(Y, H, r, Vini, mu_ini, tol, max_iter)
% An implementation of the Wiberg algorithm for the matrix-factorization Y
% -> U V' + 1 mu'
%
% [U, V] = wiberg_mean(Y, H, r) factorizes the data matrix Y (say m x n)
% into U V' + 1 mu' where U is m x r, V is n x r and mu is a n-vector. Note
% that r specifies the column size of U and V. The indicator H that has the
% same size as Y specifies non-missing/missing components, i.e., H(i, j) =
% 0 if Y(i, j) is missing and H(i, j) = 1 if Y(i, j) is non-missing.   
%
% [U, V] = wiberg_mean(Y, H, r, Vini, muini) uses Vini and muini as initial
% values of V and mu, respectively, instead of random initial values.
%
% [U, V] = wiberg_mean(Y, H, r, Vini, muini, tol, max_iter) further uses tol
% instead of the default torelance (1e-6) and max_iter instead of the
% default maximum iteration counts (100). 

if nargin < 4, 
    Vini = rand(size(Y, 2), r);
    mu_ini = rand(size(Y, 2), 1);
end
if nargin < 7, max_iter = 100;
end
if nargin < 6, tol = 1e-6;
end

%==========================================================================
% Check dimensions
%==========================================================================
m = size(Y,1);
n = size(Y,2);

% Make the vector u & v from the matrix U & V
Vt = [Vini, mu_ini].';
v = Vt(:);

%==========================================================================
% Count the number of nonmissing components
%==========================================================================
p = 0;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, p = p + 1;
        end
    end
end

%disp(sprintf('Number of nonmissing components = %d\n', p));

F = zeros(p, m*r);
G = zeros(p, n*(r+1));
y = zeros(p, 1);
mvec = zeros(p, 1);

% Make y from Y
q = 1;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, 
            y(q) = Y(i,j);
            G(q, (j-1)*(r+1)+r+1) = 1.0; 
            q = q+1;
        end
    end
end

%==========================================================================
% Start of the loop
%==========================================================================
err_last = 0.;
for iter = 1:max_iter
 
    % 2.1 Make F and m from v
    q = 1;
    for i=1:m
        for j=1:n
            if H(i, j) > 0.5, 
                F(q, (i-1)*r+1:(i-1)*r+r) = v((j-1)*(r+1)+1:(j-1)*(r+1)+r);
                mvec(q) = v((j-1)*(r+1)+r+1);
                q = q + 1;
            end
        end
    end
    
    % 2.2 Compute u minimizing |Fu + m - y|^2
    [FQ, FR] = qr(F, 0); u1 = FQ.'*(y-mvec); u = FR\u1;
    %u = F\(y - mvec);

    % 3. Check convergence 
    
    ery = y - F*u - mvec;
    err = ery.'*ery;
    disp(sprintf('[%03d] error = %f', iter, err));
    if abs(err - err_last) < tol*err, break
    end
    err_last = err;
    
    % 4.1 Make G from u
    q = 1; 
    for i = 1:m
        for j = 1:n
            if H(i, j) > 0.5,
                G(q, (j-1)*(r+1)+1:(j-1)*(r+1)+r) = u((i-1)*r+1:(i-1)*r+r);
                q = q + 1;
            end
        end
    end

    % 4.2 Determine dv minimizing |QF G dv - QF y|^2
    % QF = (I - F*inv(F.'*F)*F.')
    %FFTFinv = F*inv(F.'*F);
    QFG = G - FQ*(FQ.'*G);
    [U1, W1, V1] = svd(QFG,0);
    for i = 1:(n-r)*(r+1)
        W1(i,i) = 1.0/W1(i,i);
    end
    QFy = y-mvec - FQ*(FQ.'* (y-mvec));
    dv = V1(1:n*(r+1),1:(n-r)*(r+1))*W1(1:(n-r)*(r+1),1:(n-r)*(r+1)).'*U1(1:p,1:(n-r)*(r+1)).' * QFy;

    % 4.3 Update v
    v = v + dv;
end

% Recover U, V, mu from u and v
U = zeros(m, r);
V = zeros(n, r);
mu = zeros(n, 1);
for i=1:m
    U(i,:) = u((i-1)*r+1:(i-1)*r+r);
end
for j=1:n
    V(j,:) = v((j-1)*(r+1)+1:(j-1)*(r+1)+r);
    mu(j) = v((j-1)*(r+1)+r+1);
end



