function [U, V] = wiberg_nomean_blockcomp(Y, H, r, Uini, Vini, tol, max_iter)
% The Wiberg algorithm for the mean_free model (Y -> U V'): The block structures of the matrices are maximally used (but not so fast). 
% Y --> U V.' where U is m x r, V is n x r (Y is m x n)
% H is the indicator matrix of missing components that has the same size as Y.
% H(i, j) = either 0 (the component Y(i,j) is missing) or 1(existing)

disp('wiberg_nomean');

if nargin < 7, max_iter = 100;
end
if nargin < 6, tol = 1e-6;
end

%==========================================================================
% Check dimensions
%==========================================================================
m = size(Y,1);
n = size(Y,2);

disp(sprintf('Input data matrix(Y) = %d x %d\n', m, n));

U = Uini; %zeros(m, r);
V = Vini; %zeros(n, r);

% Make the vector u & v from the matrix U & V
Ut = U.';
u = Ut(:);
Vt = V.';
v = Vt(:);

%==========================================================================
% Count the number of nonmissing components
%==========================================================================
p = 0;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, p = p + 1;
        end
    end
end

disp(sprintf('Number of nonmissing components = %d\n', p));

F = zeros(p, m*r);
G = zeros(p, n*r);
y = zeros(p, 1);
% FG2 <- inv(F.'*F)*(F.'G)
FG2 = zeros(m*r, n*r);
zero_r = zeros(r, r);
QFG = zeros(p, n*r); % the same size as G
QFy = zeros(p, 1);

% Make y from Y
q = 1;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, 
            y(q) = Y(i,j);
            q = q+1;
        end
    end
end

etime = zeros(5,1);

%==========================================================================
% Start of the loop
%==========================================================================
err_last = 0.;
for iter = 1:max_iter
 
    % 2.1 Make F from v
    e = cputime;
    q = 1;
    for i=1:m
        for j=1:n
            if H(i, j) > 0.5, 
                F(q, (i-1)*r+1:(i-1)*r+r) = v((j-1)*r+1:(j-1)*r+r);
                q = q + 1;
            end
        end
    end
    etime(1) = etime(1) + cputime - e;
      
    % 2.2 Compute u minimizing |Fu - y|^2
    e = cputime;
    u = F\y;
    etime(2) = etime(2) + cputime - e;
    
    % 3. Check convergence 
    
    e = cputime;    
    ery = y - F*u;
    err = ery.'*ery;
    disp(sprintf('[%03d] error = %f\n', iter, err));
    if abs(err - err_last) < tol*err, break
    end
    err_last = err;
    etime(3) = etime(3) + cputime - e;
    
    % 4.1 Make G from u
    e = cputime;
    q = 1; 
    for i = 1:m
        for j = 1:n
            if H(i, j) > 0.5,
                G(q, (j-1)*r+1:(j-1)*r+r) = u((i-1)*r+1:(i-1)*r+r);
                q = q + 1;
            end
        end
    end
    etime(4) = etime(4) + cputime - e;

    % 4.2 Determine dv minimizing |QF G dv - QF y|^2
    % QF = (I - F*inv(F.'*F)*F.')
    % a. First compute FG2 = inv(F.'*F)*(F.'*G) & y2 = inv(F.'*F)*(F.'*y)
    e = cputime;
    q = 1;
    
    for i=1:m
        Ftf_bm = zeros(r, r);
        for j = 1:n
            if H(i, j) > 0.5,
                Ftf_bm = Ftf_bm + v((j-1)*r+1:(j-1)*r+r)*v((j-1)*r+1:(j-1)*r+r).';
            end
        end
        Ftf_bm = inv(Ftf_bm);
        for j=1:n
            if H(i, j) > 0.5,
                FG2((i-1)*r+1:(i-1)*r+r,(j-1)*r+1:(j-1)*r+r) = (Ftf_bm*v((j-1)*r+1:(j-1)*r+r)) * u((i-1)*r+1:(i-1)*r+r).';
                tempvr = zeros(r,1);
                for l=1:n
                    if H(i, l) > 0.5
                        tempvr = tempvr + Y(i, l) * v((l-1)*r+1:(l-1)*r+r);
                    end
                end
                QFy(q) = y(q) - v((j-1)*r+1:(j-1)*r+r).' * (Ftf_bm * tempvr);
                q = q + 1;
            else
                FG2((i-1)*r+1:(i-1)*r+r,(j-1)*r+1:(j-1)*r+r) = zero_r;
            end
        end
    end
    
    % b. Second compute QFG = G - F*FG2
    %{
    q = 1;
    QFG = G;
    for i=1:m
        for j=1:n
            if H(i, j) > 0.5,
                for l=1:n
                    QFG(q:q, (l-1)*r+1:(l-1)*r+r) = QFG(q:q, (l-1)*r+1:(l-1)*r+r) - v((j-1)*r+1:(j-1)*r+r).' * FG2((i-1)*r+1:(i-1)*r+r, (l-1)*r+1:(l-1)*r+r);
                end
                q = q + 1;
            end
        end
    end
    %}
    
    QFG = G - F*FG2;
    
    %FFTFinv = F*inv(F.'*F);
    %QFG = G - FFTFinv*(F.'*G);
    % It always holds that row >= col for QFG
    [U1, W1, V1] = svd(QFG, 0);
    for i = 1:(n-r)*r
        W1(i,i) = 1.0/W1(i,i);
    end
    %dv = QFG\QFy;
    %dv = pinv(QFG)*QFy;
    
    dv = V1(1:n*r,1:(n-r)*r)*W1(1:(n-r)*r,1:(n-r)*r)*U1(1:p,1:(n-r)*r).' * QFy;
    etime(5) = etime(5) + cputime - e;

    % 4.3 Update v
    v = v + dv;
end

disp('Complete.');

% Recover U and V from u and v
for i=1:m
    U(i,:) = u((i-1)*r+1:(i-1)*r+r);
end
for j=1:n
    V(j,:) = v((j-1)*r+1:(j-1)*r+r);
end




