function [U, V] = wiberg_nomean_naive(Y, H, r, Uini, Vini, tol, max_iter)
% The Wiberg algorithm for the mean_free model (Y -> U V'): Naive and inefficient implementation (but reasonably fast). 
% Y --> U V.' where U is m x r, V is n x r (Y is m x n)
% H is the indicator matrix of missing components that has the same size as Y.
% H(i, j) = either 0 (the component Y(i,j) is missing) or 1(existing)

disp('wiberg_nomean_instructive');

if nargin < 7, max_iter = 100;
end
if nargin < 6, tol = 1e-6;
end

%==========================================================================
% Check dimensions
%==========================================================================
m = size(Y,1);
n = size(Y,2);

disp(sprintf('Input data matrix(Y) = %d x %d\n', m, n));

U = Uini; %zeros(m, r);
V = Vini; %zeros(n, r);

% Make the vector u & v from the matrix U & V
Ut = U.';
u = Ut(:);
Vt = V.';
v = Vt(:);

%==========================================================================
% Count the number of nonmissing components
%==========================================================================
p = 0;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, p = p + 1;
        end
    end
end

%disp(sprintf('Number of nonmissing components = %d\n', p));

F = zeros(p, m*r);
G = zeros(p, n*r);
y = zeros(p, 1);

% Make y from Y
q = 1;
for i=1:m
    for j=1:n
        if H(i,j) > 0.5, 
            y(q) = Y(i,j);
            q = q+1;
        end
    end
end

%==========================================================================
% Start of the loop
%==========================================================================
err_last = 0.;
for iter = 1:max_iter
 
    % 2.1 Make F from v
    q = 1;
    for i=1:m
        for j=1:n
            if H(i, j) > 0.5, 
                F(q, (i-1)*r+1:(i-1)*r+r) = v((j-1)*r+1:(j-1)*r+r);
                q = q + 1;
            end
        end
    end
   
    % 2.2 Compute u minimizing |Fu - y|^2
    u = F\y;
    
    % 3. Check convergence   
    ery = y - F*u;
    err = ery.'*ery;
    disp(sprintf('[%03d] error = %f\n', iter, err));
    if abs(err - err_last) < tol*err, break
    end
    err_last = err;
    
    % 4.1 Make G from u
    q = 1; 
    for i = 1:m
        for j = 1:n
            if H(i, j) > 0.5,
                G(q, (j-1)*r+1:(j-1)*r+r) = u((i-1)*r+1:(i-1)*r+r);
                q = q + 1;
            end
        end
    end
    
    % 4.2 Determine dv minimizing |QF G dv - QF y|^2
    % QF = (I - F*inv(F.'*F)*F.')    
    FFTFinv = F*inv(F.'*F);
    QFG = G - FFTFinv*(F.'*G);
    QFy = y - FFTFinv*(F.'*y);
    
    % It always holds that row >= col for QFG
    [U1, W1, V1] = svd(QFG, 0);
    for i = 1:(n-r)*r
        W1(i,i) = 1.0/W1(i,i);
    end

    dv = V1(1:n*r,1:(n-r)*r)*W1(1:(n-r)*r,1:(n-r)*r)*U1(1:p,1:(n-r)*r).' * QFy;
    
    % 4.3 Update v
    v = v + dv;
end

disp('Complete.');

% Recover U and V from u and v
for i=1:m
    U(i,:) = u((i-1)*r+1:(i-1)*r+r);
end
for j=1:n
    V(j,:) = v((j-1)*r+1:(j-1)*r+r);
end




